import wx

class MainWindow(wx.Panel):
    def __init__(self, parent, id, title='', pos=(500,500), size=(800,700),individuals=None, buttons=8 ):

        wx.Panel.__init__(self, parent, wx.ID_ANY, size=size)

        self.panelSizer = wx.FlexGridSizer(cols=4, hgap=20, vgap=20)
        self.buttons = buttons
        self.popsize = len(individuals)

        # create a panel for every individual 
        for i in range(self.popsize):
            buttonFrame = ButtonFrame(self, i, size=(220,160), buttons=self.buttons, labels=individuals[i], individual=i+1)
            self.panelSizer.Add(buttonFrame)
            
        # Window sizer
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.panelSizer)

        # Radio button sizer
        rdBsizer = wx.BoxSizer(wx.HORIZONTAL)

        ranks = [`(i+1)`+'  ' for i in range(self.popsize)]

        # Best rank radio buttons
        self.rbBest = wx.RadioBox( self, -1, "Best Individual", 
                wx.DefaultPosition, wx.DefaultSize,
                ranks, 1, wx.RA_SPECIFY_ROWS)


        # Spacer
        self.sizer.Add(wx.StaticText(self, -1, ' ', size=(10,20)))
        self.sizer.Add(self.rbBest, 0, wx.ALIGN_CENTER)

        # Spacer
        self.sizer.Add(wx.StaticText(self, -1, ' ', size=(10,20)))
        # Worst rank radio buttons
        self.rbWorst = wx.RadioBox( self, -1, "Worst Individual", 
                wx.DefaultPosition, wx.DefaultSize,
                ranks, 1, wx.RA_SPECIFY_ROWS)
        self.sizer.Add(self.rbWorst, 0, wx.ALIGN_CENTER)

        self.Bind(wx.EVT_RADIOBOX, self.OnRadioBoxWorst, self.rbWorst)
        self.Bind(wx.EVT_RADIOBOX, self.OnRadioBoxBest, self.rbBest)

        self.SetSizer(self.sizer)

        self.bestInd = 0 
        self.worstInd = 0


    def OnRadioBoxBest(self, e):
        self.bestInd = self.rbBest.GetSelection()

    def OnRadioBoxWorst(self, e):
        self.worstInd = self.rbWorst.GetSelection()

    def GetBestWorst(self):
        return self.bestInd, self.worstInd


class ButtonFrame(wx.Panel):
    def __init__(self, parent, id, size=(220,300), buttons=8, labels=None, individual=0):

        wx.Panel.__init__(self, parent, -1, size=size, style=wx.RAISED_BORDER)

        self.sizer = wx.FlexGridSizer(cols=2, hgap=4, vgap=4)
        self.buttons = buttons
        self.labels = labels

        self.sizer.Add(wx.StaticText(self, -1, '%i'%individual, style=wx.ALIGN_CENTER))
        self.sizer.Add(wx.StaticText(self, -1, ''))

        buttonIds = []
        for i in range(self.buttons):
            if self.labels[i] == 'Save':
                buttonIds.append(wx.ID_SAVE)
            elif self.labels[i] == 'Save As':
                buttonIds.append(wx.ID_SAVEAS)
            elif self.labels[i] == 'OK':
                buttonIds.append(wx.ID_OK)
            elif self.labels[i] == 'Cancel':
                buttonIds.append(wx.ID_CANCEL)
            elif self.labels[i] == 'Print Preview':
                buttonIds.append(wx.ID_PREVIEW)
            elif self.labels[i] == 'Print':
                buttonIds.append(wx.ID_PRINT)
            elif self.labels[i] == 'Copy':
                buttonIds.append(wx.ID_COPY)
            elif self.labels[i] == 'Paste':
                buttonIds.append(wx.ID_PASTE)

        for i in range(self.buttons):
            widget = wx.Button(self, buttonIds[i], '')
            self.sizer.Add(widget)

        self.SetSizer(self.sizer)
