import genetic
import random

import wx
import feedback

class UIMax(genetic.Individual):
    optimization = genetic.MAXIMIZE 
    # 3 * buttons
    length = 24
    bitsperwidget = 3

    def __init__(self, chromosome=None, uiRef=None):
        self.uiRef = uiRef
        self.chromosome = chromosome or self._makechromosome(uiRef)

        self.score = None  # set during evaluation
    

#def evaluate(self, optimum=None, uiRef=None):
    def evaluate(self, best, worst):
        self.score=8
        currentButton = 0
        mypheno=[]
        bestpheno=[]
        worstpheno=[]
        # Compare with best
        for i in range(0, self.length, 3):
            mystr = `self.chromosome[i]`+`self.chromosome[i+1]`+`self.chromosome[i+2]`
            beststr = `best.chromosome[i]`+`best.chromosome[i+1]`+`best.chromosome[i+2]`
            worststr = `worst.chromosome[i]`+`worst.chromosome[i+1]`+`worst.chromosome[i+2]`
            # Decode binary button id
            mydec = int(mystr, 2)
            bestdec = int(beststr, 2)
            worstdec = int(worststr, 2)

            mypheno.append(mydec)
            bestpheno.append(bestdec)
            worstpheno.append(worstdec)

        if mypheno == bestpheno:
            self.score = best.score
        elif mypheno == worstpheno:
            self.score = worst.score
        else:
            for i in range(len(mypheno)):
                if mypheno[i] == bestpheno[i]:
                    self.score += 1

            if self.score == 8:
                for i in range(len(mypheno)):
                    if mypheno[i] == worstpheno[i]:
                        self.score -= 1

        print 'My score: '+`self.score`+ ' '+`mypheno`+ ' '+`bestpheno`+ ' '+`worstpheno`

    def _pickpivots(self, _len):
        left = random.randrange(1, _len-2)
        right = random.randrange(left, _len-1)
        return left, right
                
    def crossover(self, other):
        "override this method to use your preferred crossover method."

        child1 = self.__class__(self.chromosome)
        child2 = other.__class__(other.chromosome)

        mypheno=[]
        otherpheno=[]
        # Compare with best
        for i in range(0, self.length, 3):
            mystr = `self.chromosome[i]`+`self.chromosome[i+1]`+`self.chromosome[i+2]`
            otherstr = `other.chromosome[i]`+`other.chromosome[i+1]`+`other.chromosome[i+2]`
            # Decode binary button id
            mydec = int(mystr, 2)
            otherdec = int(otherstr, 2)

            mypheno.append(mydec)
            otherpheno.append(otherdec)

        left, right = self._pickpivots(len(mypheno))

        for i in range(left, right+1):
            fal = other.chromosome.index(self.chromosome[i])

            # Swap child 2 bits
            temp = child2.chromosome[fal]
            child2.chromosome[fal] = child2.chromosome[i]
            child2.chromosome[i] = temp

            fal = self.chromosome.index(other.chromosome[i])
            # Swap child 1 bits
            temp = child1.chromosome[fal]
            child1.chromosome[fal] = child1.chromosome[i]
            child1.chromosome[i] = temp

        return child1, child2


    # some crossover helpers ...
    def _repair(self, parent1, parent2):
        '''
        Override this method, if necessary, to fix duplicated genes.
        PMX
        '''
        pass

    
    def mutate(self, gene):
        '''
        Mutate by swapping two bits.
        '''
        swapIndex = random.randint(0,self.length-1)
        temp = self.chromosome[gene]
        self.chromosome[gene] = self.chromosome[swapIndex]
        self.chromosome[swapIndex] = temp

    def _makechromosome(self,uiRef):
        "makes a chromosome from randomly selected alleles."
        perm = [i for i in range(len(uiRef))]
        random.shuffle(perm)
        chrom=[]
        for id in perm:
            if id == 0:
                chrom.extend([0,0,0])
            else:
                temp = [0,0,0]
                for i in range(self.bitsperwidget):
                    temp[i] = id % 2
                    id = id / 2
                temp.reverse()
                chrom.extend(temp)
        return chrom

    def getPhenotype(self, uiRef):
        pheno = []
        for i in range(0, self.length, 3):
            str = `self.chromosome[i]`+`self.chromosome[i+1]`+`self.chromosome[i+2]`
            # Decode binary button id
            dec = int(str, 2)
            pheno.append(uiRef[dec])
        return pheno


class GeneticAlgorithm(genetic.Environment):
    '''
    @param size population size.
    @param kind population individual type.
    @param maxgenerations max number of generations to run.
    @optimum optimum value of evaluation function, will make the GA stop evaluations.
    '''
    def __init__(self, parent, kind, population=None, size=100, maxgenerations=100, 
                 crossover_rate=0.75, mutation_rate=0.005, optimum=None, nButtons=8, uiRef=None):
        '''
        GA constructor.
        '''
        self.parent = parent

        self.nButtons = nButtons
        self.uiRef = uiRef
        self.kind = kind
        self.size = size
        self.optimum = optimum
        # Make population
        self.population = population or self._makepopulation()

        self.crossover_rate = crossover_rate
        self.mutation_rate = mutation_rate
        self.maxgenerations = maxgenerations
        self.generation = 0

#------------------------------------------#
    def initPop(self):
        self.evaluate()
#------------------------------------------#

    def report(self):
        print "="*70
        print "generation: ", self.generation
        print "best:       ", self.best
        print "UI:         ", self.best.decodeUI(self.uiRef)

#------------------------------------------#
    def run(self, best, worst):
        if not self._goal():
            self.interpolateFitness(best, worst)
            self.report()
            self.step()
            return True
        else:
            return False

#------------------------------------------#
    def step(self):
        self.population.sort()
        self._crossover()
        self.generation += 1
#------------------------------------------#

    def _goal(self):
        return self.generation > self.maxgenerations
#------------------------------------------#

    def interpolateFitness(self, best, worst):
        # Clear all fitnesses
        for individual in self.population:
            individual.score=-1
        self.population[best].score=2*self.nButtons
        self.population[worst].score=2*self.nButtons
        for individual in self.population:
            # if individual has not fitness
            if individual.score == -1:
                individual.evaluate(self.population[best], self.population[best])
#------------------------------------------#

    def _crossover(self):
        next_population = [self.best.copy()]
        while len(next_population) < self.size:
            mate1 = self._select()
            if random.random() < self.crossover_rate:
                mate2 = self._select()
                offspring = mate1.crossover(mate2)
            else:
                offspring = [mate1.copy()]
            for individual in offspring:
                self._mutate(individual)
                next_population.append(individual)
        self.population = next_population[:self.size]
#------------------------------------------#

    def _makepopulation(self):
        return [self.kind(uiRef=self.uiRef) for individual in range(self.size)]
#------------------------------------------#
    
    def getPhenoPanel(self):
        return self.frame
#------------------------------------------#

    def evalOffspring(self):
        self.evaluate(self)

    def evaluate(self):
        # Create a list of phenotypes
        phenos=[]
        for individual in self.population:
            phenos.append(individual.getPhenotype(self.uiRef))

        # Display phenotypes to user to rank them
        self.frame = feedback.MainWindow(self.parent, -1, 'UI Evolution',  pos=(50,50), individuals=phenos,buttons=self.nButtons)

#------------------------------------------#

ID_RUN = 101
ID_EXIT = 110

class GAWindow(wx.Frame):
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, wx.ID_ANY, title, size=(900,800),pos=(50,50),
                          style=wx.DEFAULT_FRAME_STYLE|wx.NO_FULL_REPAINT_ON_RESIZE)

        self.CreateStatusBar()
        filemenu = wx.Menu()
        filemenu.Append(ID_RUN, "&Run GA", "Run the GA.")
        filemenu.AppendSeparator()
        # id, label text, status bar message
        filemenu.Append(ID_EXIT, "E&xit", "Terminate the program")

        menuBar = wx.MenuBar()
        menuBar.Append(filemenu, "&File")
        self.SetMenuBar(menuBar)
        self.Show(True)

        # Load UI reference
        file = open('uiref')
        data = file.readlines()
        self.nButtons = int(data[0].strip())
        labels = [label.strip() for label in data[1:]]

        # Create dictionary of ids to labels
        self.labelIds = dict([(i,labels[i]) for i in range(len(labels))])

        wx.EVT_MENU(self, ID_RUN,  self.OnRun )
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)

        self.emptyPanel = wx.Panel(self)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.emptyPanel)

        tempSizer = wx.BoxSizer(wx.HORIZONTAL)
        tempSizer.Add(wx.StaticText(self, -1, '', size=(15,2)))
        tempSizer.Add(wx.StaticText(self, -1, 'Population Size:   '))

        self.pop = wx.TextCtrl(self, -1, '', size=(40,25), style=wx.TE_READONLY)
        self.pop.SetValue('10')
        self.incB = wx.Button(self, wx.ID_UP, '', size=(70,30))
        self.decB = wx.Button(self, wx.ID_DOWN, '', size=(70,30))

        self.Bind(wx.EVT_BUTTON, self.OnInc, self.incB)
        self.Bind(wx.EVT_BUTTON, self.OnDec, self.decB)

        # Pop size and inc/dec size sizer
        tempSizer.Add(self.pop)
        tempSizer.Add(wx.StaticText(self, -1, ' ', size=(10,2)))
        tempSizer.Add(self.incB)
        tempSizer.Add(wx.StaticText(self, -1, ' ', size=(5,2)))
        tempSizer.Add(self.decB)

        tempSizer.Add(wx.StaticText(self, -1, '', size=(40,5)))
        # Num widgets
        tempSizer.Add(wx.StaticText(self, -1, 'Number of Widgets:  ', 
                    size=(100,40), style=wx.ALIGN_CENTER))
        self.slider = wx.Slider(self, -1, size=(100, 40), style=wx.SL_LABELS|wx.SL_AUTOTICKS)
        self.slider.SetRange(6,20)
        self.slider.SetValue(8)
        tempSizer.Add(self.slider)
        self.Bind( wx.EVT_SCROLL, self.OnWidgetNumChange, self.slider)

        self.sizer.Add(tempSizer, 0, wx.ALIGN_CENTER)

        # Add submit button
        self.submit = wx.Button(self, -1, 'Submit')
        self.sizer.Add(wx.StaticText(self, -1, ' ', size=(10,20)))
        self.sizer.Add(self.submit, 0, wx.ALIGN_CENTER)
        self.Bind(wx.EVT_BUTTON, self.OnSubmit, self.submit)

        self.submit.Enable(False)

        self.SetSizer(self.sizer)
        self.Layout()

        self.pheno = None
        self.wait = True
        self.init = False
        self.popsize=10
        self.buttonNum = 8


    def OnWidgetNumChange(self, e):
        self.buttonNum = self.slider.GetValue()

    def OnInc(self, e):
        self.popsize+=2
        self.pop.SetValue(str(self.popsize))

    def OnDec(self, e):
        if self.popsize > 4:
            self.popsize-=2
            self.pop.SetValue(str(self.popsize))


    def OnSubmit(self, e):
        best,worst = self.pheno.GetBestWorst()
        self.wait = False
        if self.init:
            ret = self.env.run(best,worst)
            if not ret:
                print 'DONE!!!!!!!!!!!!!!!!!'
                self.sizer.Add(wx.StaticText(self,-1, 'Done!!!!!!'), 0, wx.ALIGN_CENTER)
                self.Layout()
            else:
                self.sizer.Remove(self.pheno)
                self.Layout()
                self.env.evaluate()
                self.pheno = self.env.getPhenoPanel()

                self.sizer.Add(self.pheno)
                self.Layout()
            

    def Waiting(self):
        return self.wait

    def SetWait(self):
        self.wait = True

    def Init(self):
        self.env.initPop()
        self.pheno = self.env.getPhenoPanel()
        self.sizer.Add(self.pheno)
        self.submit.Enable(True)
        self.Layout()

    def OnRun(self, e):
        '''
        Runs the GA.
        '''
        self.env = GeneticAlgorithm(self, UIMax, size=self.popsize, maxgenerations=20,
                optimum=self.nButtons, uiRef=self.labelIds)
        self.Init()
        self.init=True

    def OnExit(self, e):
        '''
        Closes application.
        '''
        self.Close(True)
